from src.helpers.concatLatex import concatLatex
from src.LatexConfiguration import LatexConfiguration
from src.SyntaxTreeComponents.Placeholder.Placeholder import Placeholder
from src.SyntaxTreeComponents.Nodes.Base.TreeNode import TreeNode
from src.KeyboardMemory import KeyboardMemory
from src.GetLatex import getEditModeLatex
from src.GetLatex import getViewModeLatex
from src.SyntaxTreeComponents.Nodes.Base.LeafNode import LeafNode
from src.SyntaxTreeComponents.Nodes.Base.BranchingNode import BranchingNode
from src.SyntaxTreeComponents.Nodes.LeafNodes.StandardLeafNode import StandardLeafNode
from src.SyntaxTreeComponents.Nodes.LeafNodes.Base.PartOfNumberWithDigits import PartOfNumberWithDigits
from src.SyntaxTreeComponents.Nodes.LeafNodes.DecimalSeparatorNode import DecimalSeparatorNode
from src.SyntaxTreeComponents.Nodes.LeafNodes.DigitNode import DigitNode
from src.SyntaxTreeComponents.Nodes.BranchingNodes.StandardBranchingNode import StandardBranchingNode
from src.SyntaxTreeComponents.Nodes.BranchingNodes.AscendingBranchingNode import AscendingBranchingNode
from src.SyntaxTreeComponents.Nodes.BranchingNodes.DescendingBranchingNode import DescendingBranchingNode
from src.SyntaxTreeComponents.Nodes.BranchingNodes.RoundBracketsNode import RoundBracketsNode
from src.SyntaxTreeComponents.Nodes.BranchingNodes.MatrixNode import MatrixNode
from src.Functions.Navigation.moveRight import moveRight
from src.Functions.Navigation.moveDown import moveDown
from src.Functions.Navigation.moveLeft import moveLeft
from src.Functions.Navigation.moveUp import moveUp
from src.Functions.Insertion.insert import insert
from src.Functions.Insertion.insertWithEncapsulateCurrent import insertWithEncapsulateCurrent
from src.Functions.Selection.leaveSelectionMode import leaveSelectionMode
from src.Functions.Insertion.insertWithEncapsulateSelection import insertWithEncapsulateSelection
from src.Functions.Insertion.insertWithEncapsulateSelectionAndPrevious import insertWithEncapsulateSelectionAndPrevious
from src.Functions.Deletion.deleteCurrent import deleteCurrent
from src.Functions.Deletion.deleteSelection import deleteSelection
from src.Functions.Selection.selectLeft import selectLeft
from src.Functions.Selection.selectRight import selectRight
from src.Functions.Selection.inSelectionMode import inSelectionMode
from src.Functions.Selection.enterSelectionMode import enterSelectionMode


from typing import Optional

class LatexConfiguration:
  def __init__(self):
    self.activePlaceholderShape : str = r'\blacksquare'
    self.activePlaceholderColor : Optional[str] = None
    self.passivePlaceholderShape : str = r'\square'
    self.passivePlaceholderColor : Optional[str] = None
    self.selectionHightlightStart : str = r'\colorbox{#ADD8E6}{\(\displaystyle'
    self.selectionHightlightEnd : str = r'\)}'

  def activePlaceholderLatex(self) -> str:
    if self.activePlaceholderColor is None:
      return self.activePlaceholderShape
    else:
      return r'{\color{' + self.activePlaceholderColor + '}' + self.activePlaceholderShape + '}'
  
  def passivePlaceholderLatex(self) -> str:
    if self.passivePlaceholderColor is None:
      return self.passivePlaceholderShape
    else:
      return r'{\color{' + self.passivePlaceholderColor + '}' + self.passivePlaceholderShape + '}'


from typing import Union
from src import KeyboardMemory, LatexConfiguration, Placeholder, TreeNode

emptyKeyboardMemory = KeyboardMemory()

def getViewModeLatex(x : Union[KeyboardMemory, Placeholder, TreeNode], latexConfiguration : LatexConfiguration) -> str:
  syntaxTreeComponent = x.syntaxTreeRoot if isinstance(x, KeyboardMemory) else x
  return syntaxTreeComponent.getLatex(emptyKeyboardMemory, latexConfiguration)

def getEditModeLatex(k : KeyboardMemory, latexConfiguration : LatexConfiguration) -> str:
  return k.syntaxTreeRoot.getLatex(k, latexConfiguration)



from typing import Union
from src import Placeholder, TreeNode

class KeyboardMemory:
  def __init__(self) -> None:
    self.syntaxTreeRoot : Placeholder = Placeholder()
    self.current : Union[TreeNode, Placeholder] = self.syntaxTreeRoot
    self.selectionDiff : Union[int, None] = None
    self.inclusiveSelectionRightBorder : Union[TreeNode, None] = None
    self.inclusiveSelectionLeftBorder : Union[TreeNode, Placeholder, None] = None


from setuptools import setup

# To use a consistent encoding
from codecs import open
from os import path

root_dir = path.abspath(path.dirname(__file__))
with open(path.join(root_dir, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="mathkeyboardengine",
    version="0.1.0-alpha",
    description="MathKeyboardEngine provides the logic for a highly customizable virtual math keyboard. It is intended for use together with any LaTeX typesetting library.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/MathKeyboardEngine/MathKeyboardEngine.Python",
    author="MathKeyboardEngine",
    author_email="symbolinker@gmail.com",
    license="ISC",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: ISC License (ISCL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent"
    ],
    packages=["src"]
)